#ifndef SANDBOX_PROJECTION_H
#define SANDBOX_PROJECTION_H

#include <opencv2/opencv.hpp>
#include "beamer.h"
#include "camera.h"

class Projection{
    private:
        cv::Mat_<float> adjustingMatrix;
        // based angle to find adjustingMatrix
        double angleRotation = 0.0;
        float distanceTopSandbox;

        // resized depth frame to dst resolution
        cv::Mat_<float> resized_depth;
        // resized src to project to dst resolution
        cv::Mat_<cv::Vec3b> resized_src;
        // Buffer containing the pixels's new location when deprojected to beamer's POV
        cv::Mat_<cv::Point2i> deprojectMap;
        // Buffer indicating from where to get the pixels in the source frame to build the output
        cv::Mat_<cv::Point2i> frameMap;

        // intrinsics parameters for deprojection, which are adapted to the projection's resolution
        cv::Point2f fxy;
        cv::Point2f ppxy;

        void deprojectPixelsFromDepth(cv::Mat_<float> &depth, Camera *camera, cv::Rect mask, cv::Point3f beamer_pos, cv::Mat_<cv::Point2i> &deprojectMap, cv::Point2f fxy, cv::Point2f ppxy);
        void filterLowestDeprojectedPoints(cv::Mat_<float> &depth, cv::Mat_<cv::Point2i> &deprojectMap, cv::Mat_<cv::Point2i> &frameMap);
        void buildFrame(cv::Mat_<float> &depth, cv::Mat_<cv::Point2i> &frameMap, cv::Mat_<cv::Vec3b> &src, cv::Mat_<cv::Vec3b> &dst);
        void holeFilling(cv::Mat_<cv::Vec3b> &dst, cv::Mat_<cv::Point2i> &frameMap);
        cv::Point2i findMatchingPixel(int i, int j, float z, Camera *camera, cv::Point3f beamer_pos, cv::Point2f fxy, cv::Point2f ppxy);

    public:
        Projection();

        void setAdjustingMatrix(cv::Mat_<float> matrix){ adjustingMatrix = matrix; };
        cv::Mat_<float> getAdjustingMatrix(){ return adjustingMatrix; };
        void setAngleRotation(double angle){ angleRotation = angle; };
        double getAngleRotation(){ return angleRotation; };
        void setDistanceTopSandbox(float dist){ distanceTopSandbox = dist; };
        float getDistanceTopSandbox(){ return distanceTopSandbox; };

        cv::Point2i rotatePixel(cv::Point2i center, double angle, cv::Point2i pixel);
        cv::Point2i revertRotatePixel(cv::Point2i center, double angle, cv::Point2i rotatedPixel);
        void adjustFrame(cv::Mat_<float> &depth, cv::Mat_<cv::Vec3b> &src, cv::Mat_<cv::Vec3b> &dst, Camera *camera, cv::Point3f beamer_pos);
        void printAdjustingMatrix();

};
#endif